// file: BiDirBuffer_exdes.v
// (c) Copyright 2009 - 2011 Xilinx, Inc. All rights reserved.
// 
// This file contains confidential and proprietary information
// of Xilinx, Inc. and is protected under U.S. and
// international copyright and other intellectual property
// laws.
// 
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// Xilinx, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) Xilinx shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or Xilinx had been advised of the
// possibility of the same.
// 
// CRITICAL APPLICATIONS
// Xilinx products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of Xilinx products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
// 
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES.

//----------------------------------------------------------------------------
// SelectIO wizard example design
//----------------------------------------------------------------------------
// This example design instantiates the IO circuitry
//----------------------------------------------------------------------------

`timescale 1ps/1ps

module BiDirBuffer_exdes
   // Clock -> q modeling delay
 #(
   // width of the data for the system
   parameter sys_w = 16,
   // width of the data for the device
   parameter dev_w = 16)
  (
   output     [1:0]       PATTERN_COMPLETED_OUT,
   inout      [sys_w-1:0] DATA_TO_AND_FROM_PINS1,
   inout      [sys_w-1:0] DATA_TO_AND_FROM_PINS2,
   output  CLK_TO_PINS,
   input                  TRISTATE_OUTPUT,
   input		  TRISTATE_OUTPUT_INV,
   input                  CLK_IN,
   input                  CLK_IN1,
   input                  CLK_RESET,
   input                  IO_RESET);


   localparam             num_serial_bits = dev_w/sys_w;
   wire        clkin1;

   reg  [sys_w-1:0]  count_out;
   reg  [sys_w-1:0]  count_out1;
   reg  [sys_w-1:0]  count_out2;
   reg [num_serial_bits-1:0] local_counter;

   wire [dev_w-1:0]       data_in_to_device1;
   wire [dev_w-1:0]       data_in_to_device2;
   reg [dev_w-1:0]       data_in_to_device1_int2;
   reg [dev_w-1:0]       data_in_to_device1_int3;

   wire  [dev_w-1:0]       data_out_from_device;
     wire   [sys_w-1:0] data_out_to_pins_predelay;
   wire        clock_enable = 1'b1;
   wire   [sys_w-1:0] data_in_from_pins_int;
   wire   [sys_w-1:0] data_in_from_pins_delay;
   wire   [sys_w-1:0] tristate_predelay;

   wire                   clk_out;

   reg rst_sync;
   reg rst_sync_int;
   reg rst_sync_int1;
   reg rst_sync_int2;
   reg rst_sync_int3;
   reg rst_sync_int4;
   reg rst_sync_int5;
   reg rst_sync_int6;
   reg [1:0] pattern_completed = 2'b00;
   wire dummy;
   wire clkin1_fast;
   wire clkin1_fast_bufg;
   wire clkin1_fast_ddr;
   wire clkin1_buf;
   wire clkin1_buf_n;
   reg start_check;


   always @(posedge clk_out or posedge IO_RESET) begin
   if (IO_RESET) begin
       rst_sync <= 1'b1;
       rst_sync_int <= 1'b1;
       rst_sync_int1 <= 1'b1;
       rst_sync_int2 <= 1'b1;
       rst_sync_int3 <= 1'b1;
       rst_sync_int4 <= 1'b1;
       rst_sync_int5 <= 1'b1;
       rst_sync_int6 <= 1'b1;
    end
   else begin
       rst_sync <= 1'b0;
       rst_sync_int <= rst_sync;
       rst_sync_int1 <= rst_sync_int;
       rst_sync_int2 <= rst_sync_int1;
       rst_sync_int3 <= rst_sync_int2;
       rst_sync_int4 <= rst_sync_int3;
       rst_sync_int5 <= rst_sync_int4;
       rst_sync_int6 <= rst_sync_int5;
   end
   end





   always @(posedge clkin1_fast) begin
   if (rst_sync_int6) begin
     count_out <= 0;
     end
   else begin
     count_out <= count_out + 1'b1;
     end
   end




   always @(posedge clkin1_fast) begin
   if (rst_sync_int6) begin
     count_out1 <= 0;
     count_out2 <= 0;
   end
   else  begin
     count_out1 <= count_out;
     count_out2 <= count_out1;
   end
end

       assign data_out_from_device = count_out2;


   always @(posedge clk_out) begin
   if (rst_sync_int6) 
       pattern_completed <= 2'b00;

   else begin
     if (&data_in_to_device1_int3) begin
       pattern_completed <= 2'b11;
     end
     else begin
       if (dummy) begin
         pattern_completed <= pattern_completed;
       end
       else begin
         pattern_completed <= 2'b00;
       end
     end
   end
  end

   always @(posedge clk_out) begin
   if (rst_sync_int6) begin
     data_in_to_device1_int2 <= 0;
     data_in_to_device1_int3 <= 0;
   end
   else begin
     data_in_to_device1_int2 <= data_in_to_device1;
     data_in_to_device1_int3 <= data_in_to_device1_int2;
   end
   end



   assign dummy = (|data_in_to_device2);

  assign PATTERN_COMPLETED_OUT =  pattern_completed;




  IBUFG
      #(.IOSTANDARD ("LVCMOS33"))
   ibufg_clk_inst
     (.I          (CLK_IN1),
      .O          (clkin1_buf));

   BUFG clkin_buf_inst
    (.O (clkin1_fast),
     .I (clkin1_buf));




genvar pin_count;
  generate for (pin_count = 0; pin_count < sys_w; pin_count = pin_count + 1) begin: pins
    // Instantiate the buffers
    ////------------------------------
    IOBUF
      #(.IOSTANDARD ("LVCMOS33"))
     iobuf_inst
       (.IO         (DATA_TO_AND_FROM_PINS2[pin_count]),
        .I          (data_out_to_pins_predelay[pin_count]),
        .O          (data_in_from_pins_int[pin_count]),
        .T          (tristate_predelay[pin_count]));
   assign data_in_from_pins_delay[pin_count] = data_in_from_pins_int[pin_count];
    assign tristate_predelay[pin_count] = TRISTATE_OUTPUT_INV;

    wire data_in_to_device_int;
    (* IOB = "true" *)
    FDRE fdre_in_inst
      (.D              (data_in_from_pins_int[pin_count]),
       .C              (clkin1_fast),
       .CE             (clock_enable),
       .R              (IO_RESET),
       .Q              (data_in_to_device_int)
      );
    assign data_in_to_device2[pin_count] = data_in_to_device_int;
    wire data_out_from_device_q;
    (* IOB = "true" *)
    FDRE fdre_out_inst
      (.D              (data_out_from_device[pin_count]),
       .C              (clkin1_fast),
       .CE             (clock_enable),
       .R              (IO_RESET),
       .Q              (data_out_from_device_q)
      );
    assign data_out_to_pins_predelay[pin_count] = data_out_from_device_q;
end
endgenerate

 
   // Instantiate the IO design
   BiDirBuffer io_inst
    (
     // Connect to the sytem for inout
     .DATA_TO_AND_FROM_PINS   (DATA_TO_AND_FROM_PINS1),
     .DATA_IN_TO_DEVICE       (data_in_to_device1),
     // From the drive out to the system
     .DATA_OUT_FROM_DEVICE    (data_out_from_device),
     .CLK_TO_PINS         (CLK_TO_PINS),
     .TRISTATE_OUTPUT         (TRISTATE_OUTPUT),
     .CLK_IN                  (CLK_IN),
     .CLK_OUT                 (clk_out),
     .CLK_RESET               (CLK_RESET),
     .IO_RESET                (rst_sync_int));

endmodule
